@@ -35,6 +35,7 @@ import statistics
 import abstract
 import os
 import cfg, utils
+import webbrowser
 
 class RunExternalFailed(Exception):
     def __init__(self, v):
@@ -50,17 +51,12 @@ def run_external_program(cmdstring, wdir, argument):
     else:
         cmdline = cmdstring % argument
     wdir = os.path.normpath(wdir)
-    dir_up = wdir.count("/") * "../" + ".."
+    cur_dir = os.getcwd()
     v = cmdline.split()
     v = v[:1] + v
-    os.chdir(wdir)
     if not os.path.exists(v[0]):
         raise RunExternalFailed(v)
-        #self.m_ui.display_error_message("'%s' does not exist." %v[0],
-        #        _("Running external program failed"))
-        os.chdir(dir_up)
         return
-    os.chdir(dir_up)
     try:
         if __child_pid is not None:
             os.kill(__child_pid, signal.SIGKILL)
@@ -78,16 +74,18 @@ def run_external_program(cmdstring, wdir, argument):
         except OSError, x:
             print >> sys.stderr, "OS Error:", x
             os._exit(-1)
-            os.chdir("..")
-        os.chdir("..")
+            os.chdir(cur_dir)
+        os.chdir(cur_dir)
     else:
         __child_pid = pid
 
-
 class SolfegeApp(cfg.ConfigUtils):
-    def __init__(self, argv, ui):
+    def __init__(self, options, ui):
+        """
+        options -- command line options parsed by optparse
+        """
         cfg.ConfigUtils.__init__(self, 'solfege-app')
-        self.m_argv = argv
+        self.m_options = options
         self.m_ui = ui
         self.check_rcfile()
         self.m_teachers = {}
@@ -99,39 +97,47 @@ class SolfegeApp(cfg.ConfigUtils):
         """See default.config for rcfileversion values, meanings and
         a description of how to add config variables.
         """
-        if cfg.get_int("app/rcfileversion") > 4:
+        if cfg.get_int("app/rcfileversion") > 5:
             cfg.drop_user_config()
             return
         if cfg.get_int("app/rcfileversion") <= 1:
             if not "example-files" in cfg.get_string('config/lessoncollections'):
                 cfg.set_string('config/lessoncollections', 
                     "%s example-files" % cfg.get_string('config/lessoncollections'))
-        if cfg.get_int("app/rcfileversion") <= 2:
-            cfg.del_key("sound/commandline")
+        if cfg.get_int("app/rcfileversion") <= 5:
+            # This is more complicated that necessary to fix an old
+            # error.
+            if cfg.get_string("sound/commandline"):
+                cfg.del_key("sound/commandline")
         if cfg.get_int("app/rcfileversion") <= 3:
             cfg.set_list("config/lessoncollections",
                 cfg.get_string("config/lessoncollections").split())
-        cfg.set_int("app/rcfileversion", 4)
+        if cfg.get_int("app/rcfileversion") <= 4:
+            cfg.del_key("config/web_browser")
+        if sys.platform == 'win32':
+            if cfg.get_string('sound/wav_player'):
+                cfg.del_key('sound/wav_player')
+        cfg.set_int("app/rcfileversion", 5)
     def setup_sound(self):
         if sys.platform == 'win32' and \
                     cfg.get_string("sound/type") == "sequencer-device":
             # just in case c:\home\.solfegerc is wrong
             cfg.set_string("sound/type", "winsynth")
-        if '--no-sound' in self.m_argv \
+        if self.m_options.no_sound \
            or cfg.get_string("sound/type") == "fake-synth":
-            soundcard.initialise_using_fake_synth('--verbose-sound-init' in self.m_argv)
+            soundcard.initialise_using_fake_synth(self.m_options.verbose_sound_init)
         elif cfg.get_string("sound/type") == "winsynth":
             try:
                 soundcard.initialise_winsynth(cfg.get_int("sound/synth_number"),
-                      verbose_init='--verbose-sound-init' in self.m_argv)
+                      verbose_init=self.m_options.verbose_sound_init)
             except:#FIXME should use except: ExceptionName
                 cfg.set_int("sound/synth_number", 0)
                 soundcard.initialise_winsynth(cfg.get_int("sound/synth_number"),
-                      verbose_init='--verbose-sound-init' in self.m_argv)
+                      self.m_options.verbose_sound_init)
         elif cfg.get_string("sound/type") == "external-midiplayer":
             soundcard.initialise_external_midiplayer(
                     cfg.get_string("sound/midi_player"),
-                    verbose_init='--verbose-sound-init' in self.m_argv)
+                    verbose_init=self.m_options.verbose_sound_init)
         elif cfg.get_string("sound/type") == '':
             self.m_ui.display_error_message(
 """You should configure sound
@@ -147,7 +153,7 @@ the preferences window.
                              cfg.get_string("sound/device_file"),
                              cfg.get_int("sound/synth_number"),
                              cfg.get_string("sound/card_info"),
-                             verbose_init='--verbose-sound-init' in self.m_argv)
+                             verbose_init=self.m_options.verbose_sound_init)
             except (soundcard.SoundInitException, OSError), e:
                 self.m_sound_init_exception = e
                 soundcard.initialise_using_fake_synth(True)
@@ -241,7 +247,7 @@ It is also possible that the OS sound setup is incorrect.
                     self.m_ui.box_dict[self.m_ui.m_viewer].on_start_practise()
                 self.m_running_exercise = fn
         elif protocol == 'http':
-            os.system(self.get_string("config/web_browser") % href)
+            webbrowser.open_new(href)
         elif protocol == 'mailto':
             os.system(self.get_string("config/mua") % href)
         else:
